import jinja2 as _j2
from .loader import RelativeFileLoader


class J2Scr:
    def __init__(self, **kwargs):
        self.loader = RelativeFileLoader()
        self.environment = _j2.Environment(loader=self.loader)
        self.set_options(**kwargs)

    def __getattr__(self, name):
        if hasattr(self.loader, name):
            return getattr(self.loader, name)
        elif hasattr(self.environment, name):
            return getattr(self.environment, name)
        raise AttributeError(f"Could not find {name}")

    def set_options(self, **kwargs):
        # Check all attrs are valid
        objs = (self.loader, self.environment)
        keys = set(kwargs.keys())
        valid_keys = set.union(*(set(dir(o)) for o in objs)).intersection(keys)
        invalid_keys = keys.difference(valid_keys)
        if len(invalid_keys):
            raise AttributeError(
                f"Could not find the following in the environment or loader: {invalid_keys}")
        # Set the specified options
        kvs = [[(k, v) for k, v in kwargs.items() if hasattr(o, k)] for o in objs]
        for o, kvl in zip(objs, kvs):
            for k, v in kvl:
                setattr(o, k, v)

    def render(self, template_path, **kwargs):
        template = self.environment.get_template(template_path)
        return template.render(**kwargs)

    def render_to_file(self, template_path, out_path, mode="w", **kwargs):
        with open(out_path, mode=mode) as f:
            f.write(self.render(template_path, **kwargs))



from .j2scr import J2Scr

state = J2Scr()


def __getattr__(name):
    if hasattr(state, name):
        attr = getattr(state, name)
        return attr
    raise AttributeError(f"module {__name__} has no attribute '{name}'")



#!/usr/bin/env python3
import sys
from setuptools import setup, find_packages
from pathlib import Path

long_description = (Path(__file__).parent / "README.md").read_text()

setup(
    name="j2scr",
    version="1.0.0",
    license="MIT",
    author="Jonas Lundholm Bertelsen",
    packages=find_packages(),
    install_requires=["jinja2"],
    description="Convenience wrapper around Jinja for scripts",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jonaslb/j2scr",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
)



from jinja2.loaders import BaseLoader
from jinja2.exceptions import TemplateNotFound
from pathlib import Path


class RelativeFileLoader(BaseLoader):
    """Imitates FileSystemLoader but is always relative and allows eg `..`."""
    def __init__(self, encoding="utf-8"):
        self.encoding = encoding

    def get_source(self, environment, template):
        p = Path(template)
        if not p.is_file():
            raise TemplateNotFound(template)

        contents = open(p, mode="r", encoding=self.encoding).read()

        mtime = p.stat().st_mtime

        def uptodate():
            try:
                return p.stat().st_mtime == mtime
            except OSError:
                return False
        return contents, p.name, uptodate
